#!/usr/bin/env python3
"""CrooHtmlReportFileTable

    Jin Lee (leepc12@gmail.com) at ENCODE-DCC
"""

import os
from autouri import AutoURI


class CrooHtmlReportFileTable(object):
    HEAD = """
    <link href="https://cdnjs.cloudflare.com/ajax/libs/jquery-treetable/\
3.2.0/css/jquery.treetable.min.css" rel="stylesheet" type="text/css" />
    <link href="https://cdnjs.cloudflare.com/ajax/libs/jquery-treetable/\
3.2.0/css/jquery.treetable.theme.default.min.css" rel="stylesheet" />
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery-treetable/\
3.2.0/jquery.treetable.min.js"></script>
    <script type="text/javascript">
      $(function(){
        $("#filetable").treetable({
          expandable: true,
          onNodeCollapse: function() {},
          onNodeExpand: function() {}
        });
      });
    </script>    
    """    
    BODY = """
    <div id='file-table'><b>File table</b>
    <table id='filetable'>
      <caption>
        <a href='#' onclick="jQuery('#filetable').treetable('expandAll');\
return false;">
          Expand all</a> &nbsp&nbsp
        <a href='#' onclick="jQuery('#filetable').treetable('collapseAll');\
return false;">
          Collapse all</a>
      </caption>
    <thead><tr><th>Files</th><th>Path</th></tr></thead>
      {table_contents}
    <tbody>
    </tbody>
    </table>
    </div>
    """
    FILETABLE_TSV = 'croo.filetable.{workflow_id}.tsv'

    def __init__(self, out_dir, workflow_id):
        self._items = []
        self._out_dir = out_dir
        self._workflow_id = workflow_id

    def add(self, full_path, url, table_item):
        self._items.append((full_path, url, table_item))

    def get_html_head_str(self):
        return CrooHtmlReportFileTable.HEAD

    def get_html_body_str(self):
        html = CrooHtmlReportFileTable.BODY.format(
            table_contents=self.__make_table_contents())
        return html

    def __make_table_contents(self):
        """
        Each item has (full_path, url, table_item)
        table_item defines a hierarchy in a tree
        e.g. a/b/c with full_path=/scratch/hello.world
            and url=http://scratch.com/hello.world
        Tree    Path
        a
        +-b
          +-c   /scratch/hello.world (url as href)
        """
        # parse table_item string
        all_items = []
        data_tt_id_cache = set()
        for full_path, url, table_item in self._items:
            dir_items = table_item.split('/')
            # print(dir_items)
            for i, label in enumerate(dir_items):
                data_tt_id = '/'.join(dir_items[:i+1]).replace(' ', '-')

                if data_tt_id in data_tt_id_cache and i < len(dir_items) - 1:
                    continue
                else:
                    data_tt_id_cache.add(data_tt_id)

                if i == 0:
                    data_tt_parent_id = None
                else:
                    data_tt_parent_id = '/'.join(dir_items[:i]).replace(' ', '-')

                if i == len(dir_items) - 1:
                    if url is None:
                        path = full_path
                    else:
                        path = '<a href="{url}" target="_blank">{full_path}</a>'.format(
                            url=url,
                            full_path=full_path)
                else:
                    path = ''

                all_items.append((data_tt_id, data_tt_parent_id, label, path))

        # sort by data_tt_id but dir always comes first
        def dir_first(s):
            arr = s[0].split('/')
            if s[3] != '':  # if dir
                arr[-1] = '_' + arr[-1]
            return '/'.join(arr)

        sorted_all_items = sorted(all_items, key=lambda x: dir_first(x))

        table_contents = ''        
        for data_tt_id, data_tt_parent_id, label, path in sorted_all_items:
            table_contents += "<tr data-tt-id='{}'".format(data_tt_id)

            if data_tt_parent_id is None:
                table_contents += ">"
            else:
                table_contents += "data-tt-parent-id='{}'>".format(
                    data_tt_parent_id)

            table_contents += "<td>{label}</td><td>{path}</td></tr>\n".format(
                label=label, path=path)

        # save to TSV file
        contents = ''
        for full_path, url, table_item in self._items:
            contents += '{}\t{}\t{}\n'.format(table_item, full_path, url)
        uri_filetable = os.path.join(
            self._out_dir,
            CrooHtmlReportFileTable.FILETABLE_TSV.format(
                workflow_id=self._workflow_id))
        AutoURI(uri_filetable).write(contents)
        return table_contents



#!/usr/bin/env python3
"""CrooHtmlReportUCSCTracks, CrooHtmlReportUWTracks

Author:
    Jin Lee (leepc12@gmail.com) at ENCODE-DCC
"""

import os
import urllib.parse
from autouri import AutoURI, GCSURI, S3URI, AbsPath


class CrooHtmlReportUCSCTracks(object):
    UCSC_TRACKS_TXT = 'croo.ucsc_tracks.{workflow_id}.txt'
    UCSC_TRACKS_URL = 'croo.ucsc_tracks.{workflow_id}.url'

    UCSC_BROWSER_QUERY_POS_PARAM = '&position='
    UCSC_BROWSER_QUERY_URL = 'http://genome.ucsc.edu/cgi-bin/hgTracks?db={db}&ignoreCookie=1{extra_param}&hgct_customText={encoded}'
    UCSC_BROWSER_TEXT_FORMAT = '{track_line} bigDataUrl="{url}"\n'
    HTML_TRACK_HUB_LINK = """
<a href="{url}" target="_blank">{title}</a>
<br>
<br>"""
    HTML_TRACK_HUB_TEXT = """
<b>{title}</b>
<br>
<div style="border:2px solid silver">
<pre style="white-space:pre-wrap;word-wrap:break-word;margin:5px">
{txt}
</pre>
</div>
<br>"""

    def __init__(self, out_dir, workflow_id,
                 public_gcs=None,
                 gcp_private_key=None,
                 use_presigned_url_gcs=False,
                 use_presigned_url_s3=False,
                 duration_presigned_url_s3=None,
                 duration_presigned_url_gcs=None,
                 map_path_to_url=None,
                 ucsc_genome_db=None,
                 ucsc_genome_pos=None):
        self._out_dir = out_dir
        self._workflow_id = workflow_id
        self._public_gcs = public_gcs
        self._gcp_private_key = gcp_private_key
        self._use_presigned_url_gcs = use_presigned_url_gcs
        self._use_presigned_url_s3 = use_presigned_url_s3
        self._duration_presigned_url_s3 = duration_presigned_url_s3
        self._duration_presigned_url_gcs = duration_presigned_url_gcs
        self._map_path_to_url = map_path_to_url
        self._ucsc_genome_db = ucsc_genome_db
        self._ucsc_genome_pos = ucsc_genome_pos
        self._items = []

    def get_html_body_str(self):
        """HTML for browser track
        This HTML section provides:
            1) A plain text for UCSC genome browser &hgct_customText=
                - This is written to a text file (.txt) on the output directory
            2) An encoded URL for UCSC genome browser &hgct_customText=
            3) Clickable href link
                - Full URL is written to a text file (.url) on the output directory
        """
        if self._ucsc_genome_db is None:
            return ''
        txt = self.__make_ucsc_track_txt()
        if txt is None or txt == '':
            return ''
        if self._ucsc_genome_pos is not None:
            extra_param = CrooHtmlReportUCSCTracks.UCSC_BROWSER_QUERY_POS_PARAM \
                    + self._ucsc_genome_pos
        else:
            extra_param = ''

        # save to TXT
        uri_txt = os.path.join(
            self._out_dir,
            CrooHtmlReportUCSCTracks.UCSC_TRACKS_TXT.format(
                workflow_id=self._workflow_id))

        # localize TXT
        # long URL doesn't work
        u = AutoURI(uri_txt)
        u.write(txt)

        url_trackhub_txt_file = None
        if isinstance(u, GCSURI):
            if self._public_gcs:
                url_trackhub_txt_file = u.get_public_url()

            elif self._use_presigned_url_gcs:
                url_trackhub_txt_file = u.get_presigned_url(
                    duration=self._duration_presigned_url_gcs,
                    private_key_file=self._gcp_private_key)

        elif isinstance(u, S3URI):
            if self._use_presigned_url_s3:
                url_trackhub_txt_file = u.get_presigned_url(
                    duration=self._duration_presigned_url_s3)

        elif isinstance(u, AbsPath):
            if self._map_path_to_url:
                url_trackhub_txt_file = u.get_mapped_url(
                    map_path_to_url=self._map_path_to_url)
        html = ''

        url = CrooHtmlReportUCSCTracks.UCSC_BROWSER_QUERY_URL.format(
            db=self._ucsc_genome_db,
            extra_param=extra_param,
            encoded=urllib.parse.quote(txt))
        html += CrooHtmlReportUCSCTracks.HTML_TRACK_HUB_LINK.format(
            title='UCSC browser tracks',
            url=url)

        if url_trackhub_txt_file is not None:
            url = CrooHtmlReportUCSCTracks.UCSC_BROWSER_QUERY_URL.format(
                db=self._ucsc_genome_db,
                extra_param=extra_param,
                encoded=urllib.parse.quote(url_trackhub_txt_file))

            html += CrooHtmlReportUCSCTracks.HTML_TRACK_HUB_LINK.format(
                title='UCSC browser tracks (if the above link does not work)',
                url=url)

        html += CrooHtmlReportUCSCTracks.HTML_TRACK_HUB_TEXT.format(
            title='UCSC track hub plain text. Paste it directly to custom track edit box '
                  'on UCSC genome browser.',
            txt=txt)

        return html

    def add(self, url, track_line):
        self._items.append((url, track_line))

    def __make_ucsc_track_txt(self):
        result = ''
        for url, track_line in self._items:
            result += CrooHtmlReportUCSCTracks.UCSC_BROWSER_TEXT_FORMAT.format(
                track_line=track_line,
                url=url)
        return result



#!/usr/bin/env python3
"""CrooHtmlReport: HTML report generator for Croo

Author:
    Jin Lee (leepc12@gmail.com) at ENCODE-DCC
"""

import os
from .croo_html_report_tracks import CrooHtmlReportUCSCTracks
from .croo_html_report_file_table import CrooHtmlReportFileTable
from .croo_html_report_task_graph import CrooHtmlReportTaskGraph
from autouri import AutoURI


class CrooHtmlReport(object):
    HEAD = '@HEAD_CONTENTS'
    BODY = '@BODY_CONTENTS'
    HTML = """
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">
    <html lang="en">
      <head>
        <meta charset="utf-8">
        <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.1/\
jquery.min.js"></script>
        {head_contents}
      </head>
      <body>{body_contents}</body>
    </html>
    """.format(head_contents=HEAD, body_contents=BODY)
    REPORT_HTML = 'croo.report.{workflow_id}.html'

    def __init__(self, out_dir, workflow_id, dag,
                 task_graph_template=None,
                 public_gcs=None,
                 gcp_private_key=None,
                 use_presigned_url_gcs=False,
                 use_presigned_url_s3=False,
                 duration_presigned_url_s3=None,
                 duration_presigned_url_gcs=None,
                 map_path_to_url=None,
                 ucsc_genome_db=None,
                 ucsc_genome_pos=None):
        self._out_dir = out_dir
        self._workflow_id = workflow_id
        self._public_gcs = public_gcs
        self._gcp_private_key = gcp_private_key
        self._use_presigned_url_gcs = use_presigned_url_gcs
        self._use_presigned_url_s3 = use_presigned_url_s3
        self._duration_presigned_url_s3 = duration_presigned_url_s3
        self._duration_presigned_url_gcs = duration_presigned_url_gcs
        self._map_path_to_url = map_path_to_url
        self._ucsc_tracks = CrooHtmlReportUCSCTracks(
            out_dir=out_dir,
            workflow_id=workflow_id,
            public_gcs=public_gcs,
            gcp_private_key=gcp_private_key,
            use_presigned_url_gcs=use_presigned_url_gcs,
            use_presigned_url_s3=use_presigned_url_s3,
            map_path_to_url=map_path_to_url,
            ucsc_genome_db=ucsc_genome_db,
            ucsc_genome_pos=ucsc_genome_pos)
        self._file_table = CrooHtmlReportFileTable(
            out_dir=out_dir,
            workflow_id=workflow_id)
        self._task_graph = CrooHtmlReportTaskGraph(
            out_dir=out_dir,
            workflow_id=workflow_id,
            dag=dag,
            template_d=task_graph_template)

    def add_to_file_table(self, full_path, url, table_item):
        self._file_table.add(full_path, url, table_item)

    def add_to_ucsc_track(self, url, track_line):
        self._ucsc_tracks.add(url, track_line)

    def add_to_task_graph(self, out_var, task_name, shard_idx,
                          url, node_format, subgraph):
        self._task_graph.add(out_var, task_name, shard_idx, url,
                             node_format, subgraph)

    def save_to_file(self):
        html = CrooHtmlReport.HTML

        head = ''
        head += self._file_table.get_html_head_str()
        html = html.replace(CrooHtmlReport.HEAD, head)

        body = ''
        body += self._file_table.get_html_body_str()
        body += self._task_graph.get_html_body_str()
        body += self._ucsc_tracks.get_html_body_str()
        html = html.replace(CrooHtmlReport.BODY, body)


        # write to file and return HTML string
        uri_report = os.path.join(
            self._out_dir,
            CrooHtmlReport.REPORT_HTML.format(
                workflow_id=self._workflow_id))
        AutoURI(uri_report).write(html)
        return html


#!/usr/bin/env python3
"""CRomwellOutputOrganizer (croo): Cromwell output organizer
command line arguments helper

Author:
    Jin Lee (leepc12@gmail.com) at ENCODE-DCC
"""

import argparse
import csv
import os
import sys
from autouri import S3URI, GCSURI
from autouri import logger as autouri_logger


__version__ = '0.4.1.1'

def parse_croo_arguments():
    """Argument parser for Cromwell Output Organizer (COO)
    """
    p = argparse.ArgumentParser()
    p.add_argument(
        'metadata_json',
        help='Path, URL or URI for metadata.json for a workflow '
             'Example: /scratch/sample1/metadata.json, '
             'gs://some/where/metadata.json, '
             'http://hello.com/world/metadata.json')
    p.add_argument(
        '--out-def-json',
        help='Output definition JSON file for a WDL file corresponding to '
             'the specified metadata.json file')
    p.add_argument(
        '--method', choices=('link', 'copy'), default='link',
        help='Method to localize files on output directory/bucket. '
        '"link" means a soft-linking and it\'s for local directory only. '
        'Original output files will be kept in Cromwell\'s output '
        'directory. '
        '"copy" makes copies of Cromwell\'s original outputs')
    p.add_argument(
        '--ucsc-genome-db',
        help='UCSC genome browser\'s "db=" parameter. '
             '(e.g. hg38 for GRCh38 and mm10 for mm10)')
    p.add_argument(
        '--ucsc-genome-pos',
        help='UCSC genome browser\'s "position=" parameter. '
             '(e.g. chr1:35000-40000)')
    p.add_argument(
        '--public-gcs', action='store_true',
        help='Your GCS (gs://) bucket is public.')
    p.add_argument(
        '--use-presigned-url-s3', action='store_true',
        help='Generate presigned URLS for files on s3://.')
    p.add_argument(
        '--use-presigned-url-gcs', action='store_true',
        help='Generate presigned URLS for files on gs://. --gcp-private-key '
             'must be provided.')
    p.add_argument(
        '--gcp-private-key',
        help='Private key file (JSON/PKCS12) of a service account on '
             'Google Cloud Platform (GCP). This key will be used to make '
             'presigned URLs on files on gs://.')
    p.add_argument(
        '--duration-presigned-url-s3', type=int,
        default=S3URI.DURATION_PRESIGNED_URL,
        help='Duration for presigned URLs for files on s3:// in seconds. ')
    p.add_argument(
        '--duration-presigned-url-gcs', type=int,
        default=GCSURI.DURATION_PRESIGNED_URL,
        help='Duration for presigned URLs for files on gs:// in seconds. ')
    p.add_argument(
        '--tsv-mapping-path-to-url',
        help='A 2-column TSV file with local path prefix and corresponding '
             'URL prefix. For example, using 1-line 2-col TSV file with '
             '/var/www[TAB]http://my.server.com will replace a local path '
             '/var/www/here/a.txt to a URL http://my.server.com/here/a.txt.')
    p.add_argument(
        '--out-dir', default='.',
        help='Output directory/bucket (LOCAL OR REMOTE). '
             'This can be a local path, gs:// or s3://. ')
    p.add_argument(
        '--tmp-dir',
        help='LOCAL temporary cache directory. '
             'All temporary files for auto-inter-storage transfer will be '
             'stored here. You can clean it up but will lose all cached files '
             'so that remote files will be re-downloaded.')
    p.add_argument(
        '--use-gsutil-for-s3', action='store_true',
        help='Use gsutil for direct tranfer between GCS and S3 buckets. '
             'Make sure that you have "gsutil" installed and configured '
             'to have access to credentials for GCS and S3 '
             '(e.g. ~/.boto or ~/.aws/credientials)')
    p.add_argument(
        '--no-checksum', action='store_true',
        help='Always overwrite on output directory/bucket (--out-dir) '
             'even if md5-identical files (or soft links) already exist there. '
             'Md5 hash/filename/filesize checking will be skipped.')
    p.add_argument('-v', '--version', action='store_true',
                   help='Show version')

    group_log_level = p.add_mutually_exclusive_group()
    group_log_level.add_argument('-V', '--verbose', action='store_true',
                   help='Prints all logs >= INFO level')
    group_log_level.add_argument('-D', '--debug', action='store_true',
                   help='Prints all logs >= DEBUG level')

    if '-v' in sys.argv or '--version' in sys.argv:
        print(__version__)
        p.exit()

    if len(sys.argv[1:]) == 0:
        p.print_help()
        p.exit()
    # parse all args
    args = p.parse_args()

    if args.version is not None and args.version:
        print(__version__)
        p.exit()

    # convert to dict
    d_args = vars(args)

    check_args(d_args)
    init_dirs(d_args)
    init_autouri(d_args)

    return d_args


def check_args(args):
    """Check cmd line arguments are valid

    Args:
        args:
            dict of cmd line arguments
    """
    if args['use_presigned_url_gcs'] and args['gcp_private_key'] is None:
        raise ValueError(
            'Define --gcp-private-key to use presigned URLs on GCS'
            ' (--use-presigned-url-gcs).')

    if args['public_gcs'] and args['use_presigned_url_gcs']:
        raise ValueError(
            'Public GCS bucket (--public-gcs) cannot be presigned '
            '(--use-presigned-url-gcs and --gcp-private-key). '
            'Choose one of them.')

    if args['tmp_dir'] is None:
        pass
    elif args['tmp_dir'].startswith(('http://', 'https://')):
        raise ValueError('URL is not allowed for --tmp-dir')
    elif args['tmp_dir'].startswith(('gs://', 's3://')):
        raise ValueError('Cloud URI is not allowed for --tmp-dir')

    if args['out_dir'].startswith(('http://', 'https://')):
        raise ValueError('URL is not allowed for --out-dir')


def init_dirs(args):
    """More initialization for out/tmp directories since tmp
    directory is important for inter-storage transfer using
    Autouri

    Args:
        args:
            dict of cmd line arguments
    """
    if args['out_dir'].startswith(('gs://', 's3://')):
        if args['tmp_dir'] is None:
            args['tmp_dir'] = os.path.join(os.getcwd(), '.croo_tmp')
    else:
        args['out_dir'] = os.path.abspath(os.path.expanduser(args['out_dir']))
        os.makedirs(args['out_dir'], exist_ok=True)
        if args['tmp_dir'] is None:
            args['tmp_dir'] = os.path.join(args['out_dir'], '.croo_tmp')

    if args['tmp_dir'] is not None:
        args['tmp_dir'] = os.path.abspath(os.path.expanduser(args['tmp_dir']))


def init_autouri(args):
    """Initialize Autouri and its logger

    Args:
        args:
            dict of cmd line arguments
    """
    GCSURI.init_gcsuri(
        use_gsutil_for_s3=args['use_gsutil_for_s3'])

    # autouri's path to url mapping
    if args['tsv_mapping_path_to_url'] is not None:
        mapping_path_to_url = {}
        f = os.path.expanduser(args['tsv_mapping_path_to_url'])
        with open(f, newline='') as fp:
            reader = csv.reader(fp, delimiter='\t')
            for line in reader:
                mapping_path_to_url[line[0]] = line[1]
        args['mapping_path_to_url'] = mapping_path_to_url
    else:
        args['mapping_path_to_url'] = None

    # autouri's logger
    if args['verbose']:
        autouri_logger.setLevel('INFO')
    elif args['debug']:
        autouri_logger.setLevel('DEBUG')



#!/usr/bin/env python3
"""CrooHtmlReportTaskGraph

Author:
    Jin Lee (leepc12@gmail.com) at ENCODE-DCC
"""

import os
from copy import deepcopy
from base64 import b64encode
from graphviz import Source
from graphviz.backend import ExecutableNotFound
from autouri import AutoURI
from .croo import logger


class CrooHtmlReportTaskGraph(object):
    TASK_GRAPH_DOT = 'croo.task_graph.{workflow_id}.dot'
    TASK_GRAPH_SVG = 'croo.task_graph.{workflow_id}.svg'

    def __init__(self, out_dir, workflow_id, dag, template_d):
        """
        Args:
            out_dir:
                directory for SVG (remote or local)
            dag:
                Task graph (DAG)
            template_d:
                A template dict that will be converted to a template dot file for graphviz
                This dot file will be converted into SVG and finally be embedded in HTML
                Refer to the function caper.dict_tool.dict_to_dot_str() for details
                https://github.com/ENCODE-DCC/caper/blob/master/caper/dict_tool.py#L190
        """
        self._out_dir = out_dir
        self._workflow_id = workflow_id
        self._dag = dag
        self._template_d = template_d
        self._items = {}

    def add(self, output_name, task_name, shard_idx, url,
            node_format, subgraph):
        # node as task's output
        self._items[('output', output_name, task_name, shard_idx)] = \
                (node_format, url, subgraph)
        # node as task itself
        self._items[('task', None, task_name, shard_idx)] = \
                ('[label=\"{}\"]'.format(task_name), None, subgraph)

    def get_html_body_str(self):
        """Embed SVG into HTML
        """
        svg_contents = self.__make_svg()
        if svg_contents is None:
            return ''
        else:
            head = '<b>Task graph</b><div id=\'task-graph\'>\n'
            img = svg_contents
            tail = '</div><br>'
            return head + img + tail

    def __make_svg(self):
        """Converts a dict into a dot string and then to a SVG file
        Returns:
            An SVG string, but also saves to CrooHtmlReportTaskGraph.TASK_GRAPH_SVG
        """
        if not self._items:
            return None

        # define call back functions for node format, href, subgraph
        def fnc_node_format(n):
            if (n.type, n.output_name, n.task_name, n.shard_idx) in self._items:
                return self._items[(n.type, n.output_name, n.task_name, n.shard_idx)][0]
            else:
                return None

        def fnc_href(n):
            if (n.type, n.output_name, n.task_name, n.shard_idx) in self._items:
                return self._items[(n.type, n.output_name, n.task_name, n.shard_idx)][1]
            else:
                return None

        def fnc_subgraph(n):
            if (n.type, n.output_name, n.task_name, n.shard_idx) in self._items:
                return self._items[(n.type, n.output_name, n.task_name, n.shard_idx)][2]
            else:
                return None
        # convert to dot string
        dot_str = self._dag.to_dot(
            fnc_node_format=fnc_node_format,
            fnc_href=fnc_href,
            fnc_subgraph=fnc_subgraph,
            template=self._template_d)
        # temporary dot, svg from graphviz.Source.render
        tmp_dot = '_tmp_.dot'

        try:
            svg = Source(dot_str, format='svg').render(
                filename=tmp_dot)
        except (ExecutableNotFound, FileNotFoundError):
            logger.info(
                'Importing graphviz failed. Task graph will not be available. '
                'Check if you have installed graphviz correctly so that '
                '"dot" executable exists on your PATH. '
                '"pip install graphviz" does not install such "dot". '
                'Use apt or system-level installer instead. '
                'e.g. sudo apt-get install graphviz.')
            return None

        # save to DOT
        uri_dot = os.path.join(
            self._out_dir,
            CrooHtmlReportTaskGraph.TASK_GRAPH_DOT.format(
                workflow_id=self._workflow_id))
        AutoURI(uri_dot).write(dot_str)

        # save to SVG
        with open (svg, 'r') as fp:
            svg_contents = fp.read()
        uri_svg = os.path.join(
            self._out_dir,
            CrooHtmlReportTaskGraph.TASK_GRAPH_SVG.format(
                workflow_id=self._workflow_id))
        AutoURI(uri_svg).write(svg_contents)

        os.remove(tmp_dot)
        os.remove(svg)

        return svg_contents



#!/usr/bin/env python3
"""Croo (croo): Cromwell output organizer based on
Cromwell's metadata.json.

Author:
    Jin Lee (leepc12@gmail.com) at ENCODE-DCC
"""

import os
import sys
import logging
import json
import re
import caper
from autouri import AutoURI, AbsPath, GCSURI, S3URI, logger
from .croo_args import parse_croo_arguments
from .croo_html_report import CrooHtmlReport
from .cromwell_metadata import CromwellMetadata


logging.basicConfig(level=logging.INFO, format='%(asctime)s|%(name)s|%(levelname)s| %(message)s')
logger = logging.getLogger(__name__)


class Croo(object):
    """Cromwell output organizer (croo)

    It parses Cromwell's metadata.json to get all information about outputs
    and organize outputs as specified in output definition JSON
    """
    RE_PATTERN_INLINE_EXP = r'\$\{(.*?)\}'
    KEY_TASK_GRAPH_TEMPLATE = 'task_graph_template'
    KEY_INPUT = 'inputs'

    def __init__(self, metadata_json, out_def_json, out_dir,
                 tmp_dir,
                 soft_link=True,
                 ucsc_genome_db=None,
                 ucsc_genome_pos=None,
                 use_presigned_url_s3=False,
                 use_presigned_url_gcs=False,
                 duration_presigned_url_s3=0,
                 duration_presigned_url_gcs=0,
                 public_gcs=False,
                 gcp_private_key=None,
                 map_path_to_url=None,
                 no_checksum=False):
        """Initialize croo with output definition JSON
        Args:
            soft_link:
                DO NOT MAKE A COPY of original cromwell output
                (source) on out_dir (destination).
                Try to soft-link it if both src and dest are on local storage.
                Otherwise, original cromwell outputs will be just referenced.
        """
        self._tmp_dir = tmp_dir
        if isinstance(metadata_json, dict):
            self._metadata = metadata_json
        else:
            f = AutoURI(metadata_json).localize_on(self._tmp_dir)
            with open(f, 'r') as fp:
                self._metadata = json.loads(fp.read())
            if isinstance(self._metadata, list):
                if len(self._metadata) > 1:
                    logger.warning(
                        'Multiple metadata JSON objects '
                        'found in metadata JSON file. Taking the first '
                        'one...')
                elif len(self._metadata) == 0:
                    raise Exception('metadata JSON file is empty')
                self._metadata = self._metadata[0]
        self._out_dir = out_dir
        self._cm = CromwellMetadata(self._metadata)
        self._ucsc_genome_db = ucsc_genome_db
        self._ucsc_genome_pos = ucsc_genome_pos

        self._use_presigned_url_s3 = use_presigned_url_s3
        self._use_presigned_url_gcs = use_presigned_url_gcs
        self._duration_presigned_url_s3 = duration_presigned_url_s3
        self._duration_presigned_url_gcs = duration_presigned_url_gcs
        self._public_gcs = public_gcs
        self._gcp_private_key = gcp_private_key
        self._map_path_to_url = map_path_to_url
        self._no_checksum = no_checksum

        if isinstance(out_def_json, dict):
            self._out_def_json = out_def_json
        else:
            if out_def_json is None:
                out_def_json_file_from_wdl = self._cm.get_out_def_json_file()
                if out_def_json_file_from_wdl is None:
                    raise ValueError('out_def JSON file is not defined. '
                                     'Define --out-def-json in cmd line arg or '
                                     'add "#CROO out_def [URL_OR_CLOUD_URI]" '
                                     'to your WDL')
                out_def_json = out_def_json_file_from_wdl
            f = AutoURI(out_def_json).localize_on(self._tmp_dir)
            with open(f, 'r') as fp:
                self._out_def_json = json.loads(fp.read())

        self._task_graph = self._cm.get_task_graph()
        if Croo.KEY_TASK_GRAPH_TEMPLATE in self._out_def_json:
            self._task_graph_template = self._out_def_json.pop(Croo.KEY_TASK_GRAPH_TEMPLATE)
        else:
            self._task_graph_template = None
        if Croo.KEY_INPUT in self._out_def_json:
            self._input_def_json = self._out_def_json.pop(Croo.KEY_INPUT)
        else:
            self._input_def_json = None
        self._soft_link = soft_link

    def organize_output(self):
        """Organize outputs
        """
        report = CrooHtmlReport(
            out_dir=self._out_dir,
            workflow_id=self._cm.get_workflow_id(),
            dag=self._task_graph,
            task_graph_template=self._task_graph_template,
            public_gcs=self._public_gcs,
            gcp_private_key=self._gcp_private_key,
            use_presigned_url_gcs=self._use_presigned_url_gcs,
            use_presigned_url_s3=self._use_presigned_url_s3,
            duration_presigned_url_s3 = self._duration_presigned_url_s3,
            duration_presigned_url_gcs = self._duration_presigned_url_gcs,
            map_path_to_url=self._map_path_to_url,
            ucsc_genome_db=self._ucsc_genome_db,
            ucsc_genome_pos=self._ucsc_genome_pos)

        if self._input_def_json is not None:
            for input_name, input_obj in self._input_def_json.items():
                node_format = input_obj.get('node')
                subgraph = input_obj.get('subgraph')

                for _, node in self._task_graph.get_nodes():
                    # if node is pipeline's input
                    if node.type != 'output' or node.task_name is not None \
                            or node.output_name != input_name:
                        continue
                    full_path = node.output_path
                    shard_idx = node.shard_idx

                    if node_format is not None:
                        interpreted_node_format = Croo.__interpret_inline_exp(
                            node_format, full_path, shard_idx)
                        if subgraph is not None:
                            interpreted_subgraph = Croo.__interpret_inline_exp(
                                subgraph, full_path, shard_idx)
                        else:
                            interpreted_subgraph = None
                        report.add_to_task_graph(node.output_name,
                                                 None,
                                                 shard_idx,
                                                 full_path,
                                                 interpreted_node_format,
                                                 interpreted_subgraph)

        for task_name, out_vars in self._out_def_json.items():
            for output_name, output_obj in out_vars.items():
                path = output_obj.get('path')
                table_item = output_obj.get('table')
                ucsc_track = output_obj.get('ucsc_track')
                node_format = output_obj.get('node')
                subgraph = output_obj.get('subgraph')

                for _, node in self._task_graph.get_nodes():
                    # look at output nodes only (not a task node)
                    if task_name != node.task_name or node.type != 'task':
                        continue
                    all_outputs = node.all_outputs
                    shard_idx = node.shard_idx
                    if not all_outputs:
                        continue

                    for k, full_path, _ in all_outputs:
                        if k != output_name:
                            continue

                        target_uri = full_path
                        if path is not None:
                            interpreted_path = Croo.__interpret_inline_exp(
                                path, full_path, shard_idx)

                            au = AutoURI(full_path)
                            target_path = os.path.join(self._out_dir, interpreted_path)

                            if self._soft_link:
                                au_target = AutoURI(target_path)
                                if isinstance(au, AbsPath) and isinstance(au_target, AbsPath):
                                    au.soft_link(target_path, force=True)
                                    target_uri = target_path
                                else:
                                    target_uri = full_path
                            else:
                                target_uri = au.cp(
                                    target_path,
                                    no_checksum=self._no_checksum,
                                    make_md5_file=True)

                        # get presigned URLs if possible
                        target_url = None
                        if path is not None or table_item is not None \
                                or ucsc_track is not None or node_format is not None:
                            u = AutoURI(target_uri)

                            if isinstance(u, GCSURI):
                                if self._public_gcs:
                                    target_url = u.get_public_url()

                                elif self._use_presigned_url_gcs:
                                    target_url = u.get_presigned_url(
                                        duration=self._duration_presigned_url_gcs,
                                        private_key_file=self._gcp_private_key)

                            elif isinstance(u, S3URI):
                                if self._use_presigned_url_s3:
                                    target_url = u.get_presigned_url(
                                        duration=self._duration_presigned_url_s3)

                            elif isinstance(u, AbsPath):
                                if self._map_path_to_url:
                                    target_url = u.get_mapped_url(
                                        map_path_to_url=self._map_path_to_url)

                        if table_item is not None:
                            interpreted_table_item = Croo.__interpret_inline_exp(
                                table_item, full_path, shard_idx)
                            # add to file table
                            report.add_to_file_table(target_uri,
                                                     target_url,
                                                     interpreted_table_item)
                        if ucsc_track is not None and target_url is not None:
                            interpreted_ucsc_track = Croo.__interpret_inline_exp(
                                ucsc_track, full_path, shard_idx)
                            report.add_to_ucsc_track(target_url,
                                                     interpreted_ucsc_track)
                        if node_format is not None:
                            interpreted_node_format = Croo.__interpret_inline_exp(
                                node_format, full_path, shard_idx)
                            if subgraph is not None:
                                interpreted_subgraph = Croo.__interpret_inline_exp(
                                    subgraph, full_path, shard_idx)
                            else:
                                interpreted_subgraph = None
                            report.add_to_task_graph(output_name,
                                                     task_name,
                                                     shard_idx,
                                                     full_path if target_url is None else target_url,
                                                     interpreted_node_format,
                                                     interpreted_subgraph)
        # write to html report
        report.save_to_file()

    @staticmethod
    def __interpret_inline_exp(s, full_path, shard_idx):
        """Interpret inline expression in output defition JSON
        e.g. s can be "align/rep${i1}/${basename}"

        Args:
            full_path: full absolute path for output file
            shard_idx: tuple of scatter indices. -1 means no scatter
                       e.g. (-1, 0, 1,):
                            no scatter in main workflow
                            scatter id 0 in subworkflow
                            scatter id 1 in subsubworkflow

        Supported expressions:
            ${i} (int) : 0-based index for a main scatter loop
            ${j} (int) : 0-based index for a nested scatter loop
            ${k} (int) : 0-based index for a double-nested scatter loop
            ${basename} (str) : basename of file
            ${dirname} (str)  : dirname of file
            ${full_path} (str) : full_path of file (can be path, gs://, s3://)
            ${shard_idx} : tuple of (i, j, k, ...) with dynamic length
        """
        result = s

        if shard_idx[0] > -1:
            i = shard_idx[0]
        else:
            i = None
        if len(shard_idx) > 1 and shard_idx[1] > -1:
            j = shard_idx[1]
        else:
            j = None
        if len(shard_idx) > 2 and shard_idx[2] > -1:
            k = shard_idx[2]
        else:
            k = None
        basename = os.path.basename(full_path)
        dirname = os.path.dirname(full_path)

        while True:
            m = re.search(Croo.RE_PATTERN_INLINE_EXP, result)
            if m is None:
                break
            result = result.replace(m.group(0), str(eval(m.group(1))), 1)

        return result



def main():
    args = parse_croo_arguments()

    if args['verbose']:
        logger.setLevel('INFO')
    elif args['debug']:
        logger.setLevel('DEBUG')

    co = Croo(
        metadata_json=args['metadata_json'],
        out_def_json=args['out_def_json'],
        out_dir=args['out_dir'],
        tmp_dir=args['tmp_dir'],
        soft_link=args['method'] == 'link',
        ucsc_genome_db=args['ucsc_genome_db'],
        ucsc_genome_pos=args['ucsc_genome_pos'],
        use_presigned_url_s3=args['use_presigned_url_s3'],
        use_presigned_url_gcs=args['use_presigned_url_gcs'],
        duration_presigned_url_s3=args['duration_presigned_url_s3'],
        duration_presigned_url_gcs=args['duration_presigned_url_gcs'],
        public_gcs=args['public_gcs'],
        gcp_private_key=args['gcp_private_key'],
        map_path_to_url=args['mapping_path_to_url'],
        no_checksum=args['no_checksum'])

    co.organize_output()

    return 0


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""Directed Acylic Graph with custom hash function
You can use a non-hashable node with immutables (e.g. dict).
A DAG can be converted into Graphviz's DOT format.

Author:
    Jin Lee (leepc12@gmail.com) at ENCODE-DCC
"""

import copy
from caper.dict_tool import dict_to_dot_str


class DAG(object):
    """Directed acyclic graph with a custom hash function.

    Args:
        fnc_is_parent(n1, n2):
            function to check if n1 is a parent of n2.
        fnc_hash (optional):
            hash function to hash a node.
            this is useful when a node has a mutable object
                so that node itself is not hashable
        nodes (optional):
            list of nodes to be added to graph.

    Member variables:
        self._nodes:
            { h: n } where h = hash of a node.
        self._parents:
            { h: set([h_parent1, h_parent2, ...]) } where h = hash of a node.
        self._children:
            { h: set([h_parent1, h_parent2, ...]) } where h = hash of a node.
    """
    def __init__(self, fnc_is_parent, fnc_hash=None, nodes=None):
        self._fnc_is_parent = fnc_is_parent
        self._fnc_hash = fnc_hash
        self._nodes = {}
        self._parents = {}
        self._children = {}
        if nodes is not None:
            for n in nodes:
                self.add_node(n)

    @classmethod
    def from_dag(cls, dag):
        """Copy constructor for DAG.
        """
        return cls(is_parent=dag._is_parent, fnc_hash=dag._fnc_hash,
                   nodes=copy.copy(dag._nodes))

    def __str__(self):
        """to String.
        """
        result = '=== all nodes ===\n'
        for h, v in self._nodes.items():
            result += '{}: {}\n'.format(h, v)

        result += '\n=== parents ===\n'
        for h, v in self._parents.items():
            result += '{}: {}\n'.format(h, v)
            for _, h_ in enumerate(v):
                result += '\t{}: {}\n'.format(h_)

        result += '\n=== children ===\n'
        for h, v in self._children.items():
            result += '{}: {}\n'.format(h, v)
            for _, h_ in enumerate(v):
                result += '\t{}: {}\n'.format(h_)

        return result

    def to_dot(self, fnc_node_format, fnc_href=None, fnc_subgraph=None, template=None):
        """Converts a DAG into a Graphviz dot string.
        IMPORTANT: ONLY FORMATTED NODES WILL BE SHOWN IN THE GRAPH.

        Args:
            fnc_node_format(n):
                should starts with "[" and endswith "]"
                A single-parameter function to find a format for a node "n".
                e.g. "[shape=box, stype=filled, color=blue, label="hello"]"
            fnc_href(n) (optional):
                A single-parameter function to find a URL for a node "n".
                e.g. "http://some.where.com/rep1.bam"
                A node will not be linked if it returns None
            fnc_subgraph(n) (optional):
                A single-parameter function to find a subgraph name for a node "n".
                A node will not be associated with a subgraph if it returns None
            template (optional):
                A dict for a dot template. Useful to define a format of the whole graph.
                e.g. { "rankdir": "LR", ... }.
                key/val will be simply turned into key = val.
                If val is None then key alone without " = ".
                Refer to the function caper.dict_tool.dict_to_dot_str for details

        This function does the followings:
        1) Make a fixed dot template "digraph D {}" first
        2) Add key/val in a dict "template" to the graph D (as key = val).
        3) Find each node's format by a function "fnc_node_format".
           Only nodes with defined format will be shown in the graph, others will be hidden.
        4) Make links (edges) in a global scope of graph D.
        5) Put a node into a subgraph as defined by a function "fnc_subgraph"

        For example,
            fnc_node_format = lambda n: '[shape=box label={}]'.format(n['label']) if 'label' in n else None
            fnc_subgraph = lambda n: n['group'] if 'group' in n else None
            template = {
                "rankdir": "LR",
                "a1 [shape=box]" : None,
                "subgraph cluster_group_a" : {
                    "style": filled,
                    "color": lightgrey
                }
            }

        Then the result is:
            digraph D {
                rankdir = LR;
                a1 [shape=box];
                subgraph cluster_group_a {
                    style = filled;
                    color = lightgrey;
                    a1;
                    a2;
                }
                subgraph cluster_group_b {
                    b1;
                    b2;
                }
                a0 -> a1 -> b1 -> b2;
            }
        """
        d = copy.deepcopy(template) if template is not None else {}

        formatted_nodes = []
        for h, n in self._nodes.items():
            # wrap hash string
            quoted_h = '"' + str(h) + '"'
            format = fnc_node_format(n)
            if format is not None:
                if fnc_subgraph is not None:
                    subgraph = fnc_subgraph(n)
                    if subgraph is not None:
                        if not subgraph.startswith('subgraph '):
                            subgraph = 'subgraph ' + subgraph
                        if subgraph not in d:
                            d[subgraph] = {}
                        d[subgraph][quoted_h] = None
                if fnc_href is not None:
                    href = fnc_href(n)
                    if href is not None:
                        format = format.rstrip(']') \
                                + ' href="{url}" target="blank" tooltip="{url}"]'.format(url=href)
                formatted_nodes.append((h, format))

        for h, format in formatted_nodes:
            quoted_h = '"' + str(h) + '"'
            d['{k} {v}'.format(k=quoted_h, v=format)] = None

        # scan from root to leaf to find children
        # among candidates in formatted_nodes only

        def deepfind_child(h, candidates, depth=0):
            """DFS to find any close children in candidates.
            This doesn't visit the same branch if a child is found
            but still visits other branches to find other close children
            """
            result = []
            if h not in self._children:
                return result
            for h_child in self._children[h]:
                if h_child in candidates:
                    result.append(h_child)
                else:
                    result.extend(deepfind_child(h_child, candidates))
            return result

        # construct a parent-to-child map within formatted_nodes
        for h, format in formatted_nodes:
            quoted_h = '"' + str(h) + '"'
            children = deepfind_child(h, [i for i, _ in formatted_nodes])
            for h_child in children:
                quoted_h_child = '"' + str(h_child) + '"'
                d['{h1} -> {h2}'.format(h1=quoted_h, h2=quoted_h_child)] = None

        return dict_to_dot_str(d)

    def hash_node(self, n):
        if self._fnc_hash is None:
            return hash(n)
        else:
            return self._fnc_hash(n)

    def find_nodes(self, fnc_cond):
        """Find a list of nodes by matching condition function.

        Returns:
            [(h, n)] where h is a hash of a matched node n
        """
        result = []
        for h, n in self._nodes.items():
            if fnc_cond(n):
                result.append((h, n))
        return result

    def get_nodes(self):
        """Get a list of all nodes

        Returns:
            [(h, n)] where h is a hash of a matched node n
        """
        return self._nodes.items()

    def rm_node(self, h, recursive=False):
        """Remove a node based on hash.

        Args:
            h: hash of a node.
            recursive: remove all children nodes recursively.
        """
        self._nodes.pop(h, None)
        self._parents.pop(h, None)
        self._children.pop(h, None)
        for _, v in self._parents.items():
            if h in v:
                v.remove(h)
        for _, v in self._children.items():
            if h in v:
                v.remove(h)

    def add_node(self, n):
        """Add a node to graph.
        """
        # get hash
        h = self.hash_node(n)

        if h in self._nodes:
            # remove all links to n in parents graph
            for h_ in self._parents:
                if h == h_:
                    continue
                parents = self._parents[h_]
                if h in parents:
                    parents.remove(h)
            # remove all links to n in children graph
            for h_ in self._children:
                if h == h_:
                    continue
                children = self._children[h_]
                if h in children:
                    children.remove(h)

        self._parents[h] = set()
        self._children[h] = set()
        self._nodes[h] = n

        # update links in graph
        for h_ in self._nodes:
            if h == h_:
                continue
            n_ = self._nodes[h_]
            p = self._fnc_is_parent(n, n_)
            p_ = self._fnc_is_parent(n_, n)
            if p and p_:
                raise ValueError('Detected a cyclic link in DAG.')
            elif p:
                self._parents[h_].add(h)
                self._children[h].add(h_)
            elif p_:
                self._parents[h].add(h_)
                self._children[h_].add(h)



#!/usr/bin/env python3
"""CromwellMetadata parser.
Construct a task graph based on Cromwell's metadata.json

Author:
    Jin Lee (leepc12@gmail.com) at ENCODE-DCC
"""

import re
import json
import caper
from autouri import AutoURI
from collections import OrderedDict, namedtuple
from .dag import DAG


CMNode = namedtuple('CMNode',
    ('type', 'shard_idx', 'task_name', 'output_name', 'output_path',
     'all_outputs', 'all_inputs'))


def is_parent_cmnode(n1, n2):
    """Check if n1 is a parent node of n2.
    There are two types of nodes:
    1) task
    2) output
    """
    if n1.type not in ('task', 'output') or n2.type not in ('task', 'output'):
        raise ValueError('Unsupported CMNode type: {}.'.format(n1.type))

    if n1.type == 'task' and n2.type == 'output':
        return n1.task_name == n2.task_name and n1.shard_idx == n2.shard_idx

    elif n1.type == 'output' and n2.type == 'task':
        return n2.all_inputs is not None and \
                n1.output_path in [path for _, path, _ in n2.all_inputs]

    return False


def find_files_in_dict(d):
    files = []
    for k, v in d.items():
        maybe_files = []
        if isinstance(v, list):
            for i, v_ in enumerate(v):
                if isinstance(v_, str):
                    maybe_files.append((v_, (i,)))
                elif isinstance(v_, list):
                    for j, v__ in enumerate(v_):
                        if isinstance(v__, str):
                            maybe_files.append((v__, (i, j)))
                        elif isinstance(v__, list):
                            for k, v___ in enumerate(v__):
                                if isinstance(v___, str):
                                    maybe_files.append((v___, (i, j, k)))
        elif isinstance(v, dict):
            for _, v_ in v.items():
                if isinstance(v_, str):
                    maybe_files.append((v_, (-1,)))
        elif isinstance(v, str):
            maybe_files.append((v, (-1,)))
        for f, shard_idx in maybe_files:
            if AutoURI(f).is_valid:
                files.append((k, f, shard_idx))
    return files


class CromwellMetadata(object):
    """Construct a task DAG based Cromwell's metadata.json file
    """
    RE_PATTERN_WDL_COMMENT_OUT_DEF_JSON = \
        r'^\s*\#\s*CROO\s+out_def\s(.+)'

    def __init__(self, metadata_json, debug=False):
        self._metadata_json = metadata_json

        # input JSON
        if 'submittedFiles' in self._metadata_json:
            self._input_json = json.loads(
                self._metadata_json['submittedFiles']['inputs'],
                object_pairs_hook=OrderedDict)
            # WDL contents
            self._wdl_str = self._metadata_json['submittedFiles']['workflow']
            self._out_def_json_file = self.__find_out_def_from_wdl()
        else:
            # Would work also with sub-workflow metadata that does not
            # contain 'submittedFiles'
            self._input_json = None
            # WDL contents
            self._wdl_str = None
            self._out_def_json_file = None
        # workflow ID
        self._workflow_id = self._metadata_json['id']

        # construct an indexed DAG
        self._dag = DAG(fnc_is_parent=is_parent_cmnode)

        # parse calls to add tasks and their outputs to graph
        self.__parse_calls(self._metadata_json['calls'])

        # parse input JSON to add inputs to graph
        self.__parse_input_json()

        self._debug = debug
        if self._debug:
            print(self._dag)

    def get_workflow_id(self):
        return self._workflow_id

    def get_task_graph(self):
        return self._dag

    def get_out_def_json_file(self):
        return self._out_def_json_file

    def __parse_input_json(self):
        """Recursively parse input JSON to add input files to graph
        """
        if self._input_json is None:
            return

        for file_name, file_path, shard_idx in find_files_in_dict(self._input_json):
            # add it as an "output" without an associated task
            n = CMNode(
                type='output',
                shard_idx=shard_idx,
                task_name=None,
                output_name=file_name,
                output_path=file_path,
                all_outputs=None,
                all_inputs=None)
            self._dag.add_node(n)

    def __parse_calls(self, calls, parent_wf_name='',
                      wf_alias=None, parent_wf_shard_idx=()):
        """Recursively parse calls in metadata JSON for subworkflow
        """
        for call_name, call_list in calls.items():
            for _, c in enumerate(call_list):
                shard_idx = c['shardIndex']
                status = c['executionStatus']
                if wf_alias is None:
                    wf_name = call_name.split('.')[0]
                else:
                    wf_name = wf_alias
                task_alias = call_name.split('.')[1]

                # if it is a subworkflow, then recursively dive into it
                if 'subWorkflowMetadata' in c:
                    self.__parse_calls(
                        c['subWorkflowMetadata']['calls'],
                        parent_wf_name=parent_wf_name + wf_name + '.',
                        wf_alias=task_alias,
                        parent_wf_shard_idx=(shard_idx,))
                    continue

                task_name = parent_wf_name + wf_name + '.' + task_alias
                shard_idx = parent_wf_shard_idx + (shard_idx,)

                if 'inputs' in c:
                    in_files = find_files_in_dict(c['inputs'])
                else:
                    in_files = None

                if 'outputs' in c:
                    out_files = find_files_in_dict(c['outputs'])
                else:
                    out_files = None

                # add task itself to DAG
                n = CMNode(
                    type='task',
                    shard_idx=shard_idx,
                    task_name=task_name,
                    output_name=None,
                    output_path=None,
                    all_outputs=tuple(out_files) if out_files else None,
                    all_inputs=tuple(in_files) if in_files else None)
                self._dag.add_node(n)

                if out_files:
                    for output_name, output_path, _ in out_files:
                        # add each output file to DAG
                        n = CMNode(
                            type='output',
                            shard_idx=shard_idx,
                            task_name=task_name,
                            output_name=output_name,
                            output_path=output_path,
                            all_outputs=None,
                            all_inputs=None)
                        self._dag.add_node(n)

    def __find_out_def_from_wdl(self):
        r = self.__find_val_from_wdl(
            CromwellMetadata.RE_PATTERN_WDL_COMMENT_OUT_DEF_JSON)
        return r[0] if len(r) > 0 else None

    def __find_val_from_wdl(self, regex_val):
        result = []
        for line in self._wdl_str.split('\n'):
            r = re.findall(regex_val, line)
            if len(r) > 0:
                ret = r[0].strip()
                if len(ret) > 0:
                    result.append(ret)
        return result


def main():
    import sys
    import os
    from caper_uri import init_caper_uri

    if len(sys.argv) < 2:
        print('Usage: python cromwell_metadata.py [METADATA_JSON_FILE]')
        sys.exit(1)

    init_caper_uri(os.path.join(os.getcwd(), 'cromwell_metadata_tmp'))

    m_json_file = sys.argv[1]
    CromwellMetadata(m_json_file, debug=True)


if __name__ == '__main__':
    main()



import setuptools


with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='croo',
    version='0.4.1.1',
    scripts=['bin/croo'],
    python_requires='>=3.6',
    author='Jin Lee',
    author_email='leepc12@gmail.com',
    description='CRomwell Output Organizer',
    long_description='https://github.com/ENCODE-DCC/croo',
    long_description_content_type='text/markdown',
    url='https://github.com/ENCODE-DCC/croo',
    packages=setuptools.find_packages(exclude=['examples', 'docs']),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
    ],
    install_requires=['autouri>=0.1.2.1', 'graphviz']
)
